# -*- coding: utf-8 -*-
#   StartUp Manager - Tool for editing settings for Grub and Usplash
#   Copyright (C) 2006-2007  Jimmy Rönnholm
#
#   This program is free software; you can redistribute it and/or modify
#   it under the terms of the GNU General Public License as published by
#   the Free Software Foundation; either version 2 of the License, or
#   (at your option) any later version.
#
#   This program is distributed in the hope that it will be useful,
#   but WITHOUT ANY WARRANTY; without even the implied warranty of
#   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#   GNU General Public License for more details.
#
#   You should have received a copy of the GNU General Public License
#   along with this program; if not, write to the Free Software
#   Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA

import os
import shutil
import re
from subprocess import call

def _fix_filename_spaces(filename):
    """Prepend spaces in filename with '\' if it is not already there"""
    filename = filename.strip()
    parts = filename.split(" ")
    filename = ""
    for part in parts:
        if part == "":
            part = "\ "
        elif part[-1] != "\\":
            part += "\ "
        filename += part
    filename = filename.strip()
    if filename[-1] == "\\":
        filename = filename[:-1]
    filename = filename.strip()
    return filename

class Usplash:
    """Represent the Usplash configuration."""


    def __init__(self):
        self.binary = '/sbin/usplash'
        self.config_file = '/etc/usplash.conf'
        self.themes_directory = '/usr/lib/usplash/'
        self.alternatives_file = '/etc/alternatives/usplash-artwork.so'
        if not os.path.isfile(self.binary):
            raise SystemExit('Usplash not found.')
        if not os.path.isdir(self.themes_directory):
            os.mkdir(self.themes_directory)
        if not os.path.isfile(self.config_file):
            self.set_resolution(["1024", "768"])
        self.themes = []
        self.default_theme = ""
        self.update_initramfs = False
        self.__check_themes()

    def __check_themes(self):
        """Check for installed themes."""
        self.themes = []
        current_usplash = ""
        name = self.themes_directory + "usplash-artwork.so"
        if os.path.islink(name):
            current_usplash_path = os.path.realpath(name)
            slash = current_usplash_path.rfind("/")
            if slash != -1:
                path = current_usplash_path[:slash + 1]
                if path != self.themes_directory:
                    self.add_theme(current_usplash_path)
                current_usplash = current_usplash_path[slash + 1:]

        usplash_files = os.listdir(self.themes_directory)
        for usplash_file in usplash_files:
            end = usplash_file[usplash_file.rfind("."):]
            if end == ".so" and usplash_file != "usplash-artwork.so":
                self.themes.append(usplash_file[:-3])
                if usplash_file == current_usplash:
                    self.default_theme = usplash_file[:-3]

    def set_resolution(self, resolution):
        """Set resolution.

        Change /etc/usplash.conf based on 
        list resolution[0] = xres and [1] = yres.
        """
        xres = resolution[0]
        yres = resolution[1]
        output_file = open(self.config_file, 'w')
        output_file.write("# Usplash configuration file\n")
        output_file.write("xres=" + xres + "\n")
        output_file.write("yres=" + yres + "\n")
        output_file.close()
        self.update_initramfs = True

    def get_active_theme(self):
        """Return the name of the currently active Usplash theme."""
        return self.default_theme

    def set_active_theme(self, name):
        """Set the active Usplash theme to name."""
        filename = name + ".so"
        for usplash_file in os.listdir(self.themes_directory):
            if filename == usplash_file:
                filename = _fix_filename_spaces(filename)
                call("ln -sf " + self.alternatives_file + " " + 
                          self.themes_directory + "usplash-artwork.so")
                command = '/usr/sbin/update-alternatives --set ' + \
                          'usplash-artwork.so /usr/lib/usplash/' + filename
                call(command)
                self.default_theme = name
                self.update_initramfs = True

    def get_all_themes(self):
        """Return a list of the names on the available themes."""
        return self.themes

    def add_theme(self, path):
        """Add theme from path."""
        filename = path[path.rfind("/") + 1:]
        filename = _fix_filename_spaces(filename)
        if filename[-3:] == ".so" and filename != "usplash-artwork.so":
            shutil.copy(path, self.themes_directory)
            command = '/usr/sbin/update-alternatives '+ \
                      '--install /usr/lib/usplash/usplash-artwork.so ' + \
            'usplash-artwork.so ' + self.themes_directory + filename + ' 10'
            call(command)
            self.__check_themes()

    def remove_theme(self, name):
        """Remove theme name."""
        name = name + ".so"
        if name != "usplash-artwork.so":
            filename = self.themes_directory + name
            os.remove(filename)
            self.__check_themes()

    def do_shutdown_tasks(self):
        """Does any post-config tasks necessary."""
        if self.update_initramfs:
            call('update-initramfs -k $(uname -r) -u')

